@@ -111,6 +111,8 @@ typedef struct _raa_db_access {
 	int genbank, embl, swissprot, nbrf;
 	int nseq, longa, maxa;
 	int L_MNEMO, WIDTH_SP, WIDTH_KW, WIDTH_SMJ, WIDTH_AUT, WIDTH_BIB, ACC_LENGTH, SUBINLNG, lrtxt, VALINSHRT2;
+	char *version_string;
+        int maxlists; /* max # of possible lists */
 	raa_node **sp_tree; /* NULL or the full taxonomy tree */
 	int max_tid; /* largest correct taxon ID value */
 	int *tid_to_rank; /* NULL or tid-to-rank table */
