@@ -13,7 +13,7 @@ void my_colorchooser_callback(Fl_Widget *obj, void *data);
 void slider_cb(Fl_Widget *obj, void *data);
 void ignore_cb(Fl_Widget *obj, void *data);
 void accept_cb(Fl_Widget *obj, void *data);
-void permanent_cb(Fl_Widget *obj, void *data);
+void permanent_cb(void *data);
 void reset_color_callback(Fl_Widget *obj, void *data);
 void accept_color_callback(Fl_Widget *obj, void *data);
 void reset_callback(Fl_Widget *obj, void *data);
@@ -67,7 +67,7 @@ public:
 void custom_callback(Fl_Widget *obj, void *data)
 {
 static Fl_Window *win = NULL;
-static Fl_Button *permanent, *accept;
+static Fl_Button *permanent, *accept, *ignore;
 static coloraas *mycoloraas;
 int x, y, w, h, c;
 SEA_VIEW *view = (SEA_VIEW *)obj->user_data();
@@ -242,17 +242,15 @@ if(win == NULL) {
 	permanent->color(FL_LIGHT2);
 	w = 45;
 	y += 10;
-	Fl_Button *ignore = new Fl_Button(win->w() - w - 5,y,w,20, "Close");
+	ignore = new Fl_Button(win->w() - w - 5,y,w,20, "Close");
 	ignore->shortcut(FL_COMMAND | 'w');
 	ignore->color(FL_LIGHT2);
-	ignore->callback(ignore_cb);
 	win->end();
 	win->resizable(NULL);
 	}
 
 mycoloraas->user_data(view);
 accept->callback(accept_cb, mycoloraas);
-permanent->callback(permanent_cb, mycoloraas);
 bgrnd->value( view->DNA_obj->color() );
 bgrnd->color( view->DNA_obj->color() );
 s_bgrnd->value( view->DNA_obj->color2() );
@@ -323,7 +321,15 @@ win->show();
 #ifndef MICRO
 win->hotspot(win);
 #endif
-return;
+  while (TRUE) {
+    Fl_Widget* obj = Fl::readqueue();
+    if (!obj) Fl::wait();
+    else if (obj == ignore || obj == permanent || obj == win) {
+      if (obj == permanent) permanent_cb(mycoloraas);
+      win->hide();
+      break;
+    }
+  }
 }
 
 
@@ -394,7 +400,8 @@ else item->clear();
 item = menu_props + props_parts->inverted;
 view->inverted_colors =! inverted->value(); /* because callback toggles it */
   view->menubar->value(item);
-  item->callback()(view->menubar, (void*)props_parts->inverted);
+  Fl_Callback1* cb = (Fl_Callback1*)item->callback();
+  cb(view->menubar, props_parts->inverted);
 if(view->inverted_colors) item->set();
 else item->clear();
   
@@ -428,14 +435,14 @@ fl_reset_cursor(view->dnawin);
 }
 
 
-void permanent_cb(Fl_Widget *obj, void *data)
+void permanent_cb(void *data)
 {
 char txt[150], *p;
 int v, i;
 uchar r, g, b;
 coloraas *mycoloraas = (coloraas *)data;
 
-accept_cb(obj, data);
+accept_cb(NULL, data);
 
 v = (int)(bgrnd->value() + 0.5);
 sprintf(txt, "%d", v );
@@ -495,7 +502,6 @@ set_res_value("codoncolors", txt);
 set_res_value("sequencefontsize", fontsize->text());
 
 save_resources();
-obj->window()->hide();
 }
 
 
