@@ -822,34 +822,39 @@ int process_block_trees(FILE *in, char ***ptrees, char ***pnames)
 	trees = (char **)calloc(maxi, sizeof(char *));
 	names = (char **)calloc(maxi, sizeof(char *));
 	int count = 0;
-	while(TRUE) {
-		p = next_token(in);
-		if(is_block_end(p)) break;
-		if(strcmp(p, "TRANSLATE") == 0) {
-		  ntaxa = process_translate(in, &taxnames);
+  while(TRUE) {
+    p = next_token(in);
+    if(is_block_end(p)) break;
+    if(strcmp(p, "TRANSLATE") == 0) {
+      ntaxa = process_translate(in, &taxnames);
 		  }
-		if(strcmp(p, "TREE") == 0) {
-			if(count >= maxi) {
-				maxi += 5;
-				trees = (char **)realloc(trees, maxi * sizeof(char *));
-				names = (char **)realloc(names, maxi * sizeof(char *));
-				}
-			names[count] = strdup(next_token(in));
-			next_token(in); //skip =
-			while ( isspace(c=fgetc(in)) ) ; // skip white space
-			trees[count] = (char *)malloc(500000);
-			p = trees[count];
-			*p++ = c; // first char (can be '[' for tree header)
-			while(TRUE) {
-			  *p = fgetc(in);
-			  if(*p == '\n' || *p == '\r') continue;
-			  if(*p++ == ';') break;
-			  }
-			*p = 0;
-			trees[count] = (char *)realloc(trees[count] , p - trees[count] + 1);
-		  if (ntaxa > 0) trees[count] = replace_ranks_by_names(trees[count], ntaxa, taxnames);
-			count++;
-			}
+    if(strcmp(p, "TREE") == 0) {
+      if(count >= maxi) {
+        maxi += 5;
+        trees = (char **)realloc(trees, maxi * sizeof(char *));
+        names = (char **)realloc(names, maxi * sizeof(char *));
+      }
+      names[count] = strdup(next_token(in));
+      next_token(in); //skip =
+      while ( isspace(c=fgetc(in)) ) ; // skip white space
+      trees[count] = (char *)malloc(500000);
+      p = trees[count];
+      do {
+        *p++ = c; // first char (can be '[' for tree header)
+        while(TRUE) {
+          *p = fgetc(in);
+          if(*p == '\n' || *p == '\r') continue;
+          if(*p++ == ';') break;
+        }
+        do c = fgetc(in); while (isspace(c));
+      }
+      while (c == '(' || c == '[');
+      ungetc(c, in);
+      *p = 0;
+      trees[count] = (char *)realloc(trees[count] , p - trees[count] + 1);
+      if (ntaxa > 0) trees[count] = replace_ranks_by_names(trees[count], ntaxa, taxnames);
+      count++;
+    }
 		}
 	trees = (char **)realloc(trees, count * sizeof(char *));
 	names = (char **)realloc(names, count * sizeof(char *));
