@@ -63,48 +63,6 @@ extern void win32_PDF_delete(PDF *p);
 #define PDF_delete win32_PDF_delete
 #endif // WIN32
 
-
-class Fl_PDF_Graphics_Driver : public Fl_Graphics_Driver {
-private:
-  PDF *pdf;
-  int pdf_font;
-  const char *encoding;
-  bool in_page;
-  int tr_x, tr_y;
-public:
-  Fl_PDF_Graphics_Driver();
-protected:
-  void rect(int x, int y, int w, int h);
-  void rectf(int x, int y, int w, int h);
-  void line_style(int style, int width, char *dashes=0);
-  void line(int x1, int y1, int x2, int y2);
-  void xyline(int x, int y, int x1);
-  void xyline(int x, int y, int x1, int y2);
-  void yxline(int x, int y, int y1);
-  void yxline(int x, int y, int y1, int x2);
-  void font(int f, int s);
-  void draw(const char *str, int n, int x, int y);
-  void draw(const char*, int, float, float) ;
-  void draw(int, const char*, int, int, int) ;
-  void rtl_draw(const char*, int, int, int) ;
-  void color(uchar r, uchar g, uchar b);
-  void color(Fl_Color c);
-  void push_clip(int x, int y, int w, int h) ;
-  void pop_clip();
-  void draw_image(const uchar*, int, int, int, int, int, int) ;
-  void draw_image_mono(const uchar*, int, int, int, int, int, int) ;
-  void draw_image(void (*)(void*, int, int, int, uchar*), void*, int, int, int, int, int) ;
-  void draw_image_mono(void (*)(void*, int, int, int, uchar*), void*, int, int, int, int, int) ;
-  void draw(Fl_RGB_Image*, int, int, int, int, int, int) ;
-  void draw(Fl_Pixmap*, int, int, int, int, int, int) ;
-  void draw(Fl_Bitmap*, int, int, int, int, int, int) ;
-  double width(const char*, int) ;
-  int height() ;
-  int descent() ;
-friend class Fl_PDF_File_Device;
-};
-
-
 Fl_PDF_File_Device::Fl_PDF_File_Device()
 {
   driver(new Fl_PDF_Graphics_Driver);
