@@ -6,6 +6,10 @@
 #include <FL/Fl_Graphics_Driver.H>
 #endif
 
+#if defined(WIN32)
+#include <windows.h>
+#endif
+
 #if !(defined(__APPLE__) || defined(WIN32)) && 100*FL_MAJOR_VERSION + 10*FL_MINOR_VERSION  + FL_PATCH_VERSION == 130
 // for X11 under 1.3.0 only
 #include <FL/Fl_Printer.H>
@@ -161,10 +165,8 @@ typedef Fl_PDF_File_Device Fl_PDF_or_PS_File_Device;
 #endif
 #endif
 
-#if (defined(__APPLE__) || defined(WIN32))
-
+#if defined(__APPLE__)
 class Copy_Surface : public Fl_Surface_Device {
-#ifdef __APPLE__
   bool use_pict;
   CFMutableDataRef pdfdata;
   CGContextRef oldgc;
@@ -178,18 +180,35 @@ class Copy_Surface : public Fl_Surface_Device {
   void prepare_copy_pdf_and_pict(int w, int h);
   void complete_copy_pdf_and_pict();
   void init_PDF_context(int w, int h);
-#else
-  HDC oldflgc;
-#endif
 public:
-#ifdef __APPLE__
   Copy_Surface(bool use_pict, int w, int h);
-#else
-  Copy_Surface(int w, int h);
-#endif
   ~Copy_Surface();
 };
-#endif
+#endif // __APPLE__
+
+#if defined(WIN32)
+
+#if 100*FL_MAJOR_VERSION + 10*FL_MINOR_VERSION + FL_PATCH_VERSION >= 134 // FLTK after 1.3.4
+
+#include <FL/Fl_Copy_Surface.H>
+class Copy_Surface : public Fl_Copy_Surface {
+public:
+  Copy_Surface(int w, int h) : Fl_Copy_Surface(w, h) {}
+  ~Copy_Surface() {}
+};
+
+#else
+
+class Copy_Surface : public Fl_Surface_Device {
+  HDC oldflgc;
+public:
+  Copy_Surface(int w, int h);
+  ~Copy_Surface();
+};
+
+#endif // FLTK after 1.4
+
+#endif // WIN32
 
 
 #endif // PDF_OR_PS_H
