@@ -204,6 +204,9 @@ typedef struct _SEA_VIEW {
 		  0: normal; 
 		  >= 1: 1 seul seq ecrite */
   int mod_cursor; /* FALSE: cursor ancien ignore; TRUE: ancien efface */
+#if (100*FL_MAJOR_VERSION + FL_MINOR_VERSION >= 104) && !defined(__APPLE__)
+  float scale; /* scaling factor used to compute char_width */
+#endif
   float char_width; /* largeur en pixels des caracteres */
   int line_height; /* hauteur en pixels des lignes */
   int cursor_seq, cursor_site; /* position courante du curseur */
@@ -319,6 +322,7 @@ extern char *save_alignment_or_region(const char *fname, char **seq,
 extern Fl_Window *load_alignment_file(SEA_VIEW *view, char *filename, 
 	const char *message, known_format file_format, int doing_dnd);
 extern int is_a_protein_seq(char *seq);
+extern int is_a_protein_alignment(SEA_VIEW *view);
 extern char *get_full_path(const char *fname);
 extern void draw_region_background(SEA_VIEW *view, int f_seq0, int l_seq0);
 extern void load_seq_dialog(SEA_VIEW *view);
