@@ -16,6 +16,9 @@ struct noeud {
 	struct noeud *v1,*v2,*v3;
 	char *nom;
 	int rank;
+	// Branches may be associated with id
+	// And this may be used in some cases
+	int br_id1, br_id2, br_id3;
 };
 
 struct nom {
@@ -130,5 +133,7 @@ struct FD_nj_plot {
 };
 
 extern Fl_Window *treedraw(char *tree, SEA_VIEW *view, const char *name, int from_tree_menu, int count = 1);
+char *get_next_tree(FD_nj_plot *fd_nj_plot);
+void free_tree(FD_nj_plot *fd_nj_plot);
 
 #endif // TREEDRAW_H
