@@ -3,6 +3,7 @@
 #endif
 #include "seaview.h"
 #include "treedraw.h"
+#include "tbe.h"
 #include <FL/Fl_Round_Button.H>
 #include <FL/Fl_Check_Button.H>
 #include <FL/Fl_Select_Browser.H>
@@ -43,7 +44,7 @@ void distance_method_dialog(SEA_VIEW *view);
 matrix *run_calc_dist_matrix(allseq *seqs, int distkind, int protein, char **lwlseqs, int in_bootstrap, char **p_err_mess, int no_gui);
 char *run_distance_method(SEA_VIEW *view, int distkind, int remove_all_gaps, int use_bionj, int use_bootstrap,
 			 int replicates, Fl_Box *box, const char *distance_name, int using_kaks,
-			 const char* distancefname, const char *user_tree, int no_gui, void (*alert)(const char*, ...), int keep_b_trees );
+			  const char* distancefname, const char *user_tree, int no_gui, void (*alert)(const char*, ...), int keep_b_trees, int compute_tbe );
 char *bootstrap_reformat(char *tree, int replicates);
 matrix *Obs_Dist(allseq *data, model *mod);
 matrix *Kimura_p_Dist(allseq *data);
@@ -221,12 +222,17 @@ static void distance_callback(Fl_Widget *wgt, void *data)
 
 static void dist_b_cb(Fl_Widget *wgt, void *data)
 {
+  Fl_Button **buttons = (Fl_Button **)data;
   Fl_Button *me = (Fl_Button*)wgt;
-  Fl_Button *target = (Fl_Button*)data;
-  if (me->value()) target->activate();
-  else {
-    target->deactivate();
-    target->value(0);
+  for (int i=0; i<2; i++){
+    Fl_Button *target = buttons[i];
+    if (target){
+      if (me->value()) target->activate();
+      else {
+        target->deactivate();
+        target->value(0);
+      }
+    }
   }
 }
 
@@ -249,10 +255,11 @@ void distance_method_dialog(SEA_VIEW *view)
 	static Fl_Button *interrupt;
 	static Fl_Return_Button *go;
 	static int started, maxchoice;
-  static Fl_Check_Button *keep_b_trees;
+	static Fl_Check_Button *keep_b_trees;
+	static Fl_Check_Button *compute_tbe;
 	if(first) {
 		first = FALSE;
-		w = new Fl_Window(250, 240);
+		w = new Fl_Window(270, 265);
 		w->label("Distance analysis");
 		w->set_modal();
 		nj = new Fl_Round_Button(3, 3, 60, 20, "NJ");
@@ -264,7 +271,7 @@ void distance_method_dialog(SEA_VIEW *view)
 		distance = new Fl_Choice(90, nj->y() + nj->h() + 15, 100, 20, "Distance");
 		distance->align(FL_ALIGN_LEFT);
 		ignore_all_gaps = new Fl_Check_Button(160, distance->y() + distance->h() + 5, 50, 20, 
-						      "ignore all gap sites");
+						      "Ignore all gap sites");
 		ignore_all_gaps->value(1);
 		ignore_all_gaps->align(FL_ALIGN_LEFT);
 		int y = ignore_all_gaps->y() + ignore_all_gaps->h() + 25;
@@ -274,10 +281,19 @@ void distance_method_dialog(SEA_VIEW *view)
 		char tmp[10];
 		sprintf(tmp, "%d", def_replicates);
 		replicates->value(tmp);
-    keep_b_trees = new Fl_Check_Button(3, bootstrap->y() + bootstrap->h() + 5, 180, 20, "Show bootstrap trees");
-    keep_b_trees->deactivate();
-    bootstrap->callback(dist_b_cb, keep_b_trees);
-    bootstrap->when(FL_WHEN_CHANGED);
+
+		compute_tbe = new Fl_Check_Button(3, bootstrap->y() + bootstrap->h() + 5, 280, 20, "Transfer Bootstrap Expectation method");
+		compute_tbe->deactivate();
+
+		keep_b_trees = new Fl_Check_Button(3, compute_tbe->y() + compute_tbe->h() + 5, 180, 20, "Show bootstrap trees");
+		keep_b_trees->deactivate();
+
+    static Fl_Check_Button *buttons[2];
+		buttons[0]=compute_tbe;
+		buttons[1]=keep_b_trees;
+		bootstrap->callback(dist_b_cb, buttons);
+		bootstrap->when(FL_WHEN_CHANGED);
+		
 		box = new Fl_Box(3, keep_b_trees->y() + keep_b_trees->h() + 5, 150, 20, "");
 		usertree = new Fl_Round_Button(nj->x(), box->y() + box->h() + 15, 85, nj->h(), "User tree:");
 		usertree->type(FL_RADIO_BUTTON);
@@ -362,7 +378,7 @@ void distance_method_dialog(SEA_VIEW *view)
 	  char *tree = run_distance_method(view, distance_choice, ignore_all_gaps->value(), bionj->value(), 
 						  bootstrap->value(), def_replicates, box, distance->text(), using_kaks,
 			      savetofile->value()?"":NULL, usertree->value() ? view->trees[treechoice->value()] : NULL,
-			      false, fl_alert, keep_b_trees->value());
+					   false, fl_alert, keep_b_trees->value(), compute_tbe->value());
 	  const char *choice = bionj->value()? "BioNJ_tree" : "NJ_tree";
 	  const char *n = extract_filename(view->masename);
 	  char *title = NULL;
@@ -438,7 +454,7 @@ matrix *run_calc_dist_matrix(allseq *seqs, int distkind, int protein, char **lwl
 
 char *run_distance_method(SEA_VIEW *view, int distkind, int remove_all_gaps, int use_bionj, int use_bootstrap,
 	int replicates, Fl_Box *box, const char *distance_name, int using_kaks, const char* distancefname,
-	const char *user_tree, int no_gui, void (*alert)(const char*, ...), int keep_b_trees )
+			  const char *user_tree, int no_gui, void (*alert)(const char*, ...), int keep_b_trees, int compute_tbe )
 {
 	char *display_tree, *bootstrap_trees = NULL, *err_mess, tree_label[100]="";
 	int total = 0, i, j, lbtrees;
@@ -511,7 +527,7 @@ char *run_distance_method(SEA_VIEW *view, int distkind, int remove_all_gaps, int
 	phyml_mat->tree = Make_Tree_From_Scratch(phyml_seqs->n_otu, phyml_seqs);
 	Bionj(phyml_mat);
 	sprintf(tree_label, "%s %d sites %s", use_bionj ? "BioNJ" : "NJ",
-			phyml_seqs->clean_len, distance_name );
+		phyml_seqs->clean_len, distance_name);
 	if (use_bootstrap) {
 		char *full_tree = Write_Tree(phyml_mat->tree);
 		Free_Tree(phyml_mat->tree);
@@ -532,7 +548,7 @@ char *run_distance_method(SEA_VIEW *view, int distkind, int remove_all_gaps, int
 				phyml_mat->tree = Make_Tree_From_Scratch(phyml_seqs->n_otu, phyml_seqs);
 				Bionj(phyml_mat);
 				char *one_replicate_tree = Write_Tree(phyml_mat->tree);
-        if (keep_b_trees) {
+        if (keep_b_trees || compute_tbe) {
           if (r == 0) {
             lbtrees = replicates * (strlen(one_replicate_tree) + 1);
             bootstrap_trees = (char *)malloc(lbtrees + 1);
@@ -564,7 +580,7 @@ char *run_distance_method(SEA_VIEW *view, int distkind, int remove_all_gaps, int
 		}
 		if(total > 0) {
 			display_tree = finish_add_bootstrap(total);
-			sprintf(tree_label + strlen(tree_label), " %d repl.", total);
+			sprintf(tree_label + strlen(tree_label), " %d repl.%s", total, compute_tbe ? " (TBE)" : "" );
 			}
 		else display_tree = NULL;
 	}
@@ -581,12 +597,15 @@ char *run_distance_method(SEA_VIEW *view, int distkind, int remove_all_gaps, int
 	  if( !using_kaks) alert("Can't compute distances because sequences are too divergent.\nAre they aligned?");
 	  return NULL;
 	}
+	if (bootstrap_trees && compute_tbe) {
+	  display_tree=TBE_Compute(display_tree,bootstrap_trees);
+	}
 	//add label to tree
 	char *tree = (char *)malloc(strlen(tree_label) + strlen(display_tree) + 4);
 	sprintf(tree, "[%s] %s", tree_label, display_tree);
 	free(display_tree);
 	if (box) box->window()->hide();//necessary under X11 so tree is in foreground
-  if (bootstrap_trees) {
+  if (bootstrap_trees && keep_b_trees) {
     treedraw(bootstrap_trees, view, "bootstrap trees", FALSE);
   }
   return tree;
@@ -733,17 +752,21 @@ Fl_Round_Button *g_no; Fl_Choice *g_cats; Fl_Round_Button *a_est; Fl_Float_Input
 Fl_Round_Button *nni_b, *spr_b, *nni_spr_b;
 Fl_Round_Button *u_bionj, *u_menutree, *u_random;
 Fl_Choice *u_choice;
-Fl_Check_Button *u_optimize, *u_quiet, *b_keep_trees;
+Fl_Check_Button *u_optimize, *u_quiet, *b_keep_trees, *b_tbe;
 Fl_Int_Input *u_random_count;
 char *phyml_path;
 
 static void support_cb(Fl_Widget *wid, void *data)
 {
   Fl_Button *b = (Fl_Button*)data;
-  if (b->value()) b_keep_trees->activate();
+  if (b->value()) {
+    b_keep_trees->activate();
+    b_tbe->activate();
+  }
   else {
     b_keep_trees->deactivate();
     b_keep_trees->value(0);
+    b_tbe->deactivate();
   }
 }
 
@@ -831,7 +854,7 @@ void phyml_dialog(SEA_VIEW *view)
 		}
 	y += mymodel->h() + 25;
 	//bootstrap
-	Fl_Group *b_group = new Fl_Group(0, y, w->w(), 75, "Branch Support");
+	Fl_Group *b_group = new Fl_Group(0, y, w->w(), 95, "Branch Support");
 	b_group->box(FL_ROUNDED_BOX);
 	b_group->align(FL_ALIGN_TOP|FL_ALIGN_CENTER);
 	y += 5;
@@ -846,9 +869,14 @@ void phyml_dialog(SEA_VIEW *view)
 	b_count = new Fl_Int_Input(b_yes->x() + b_yes->w() + 5, y, 50, 20, "replicates");
 	b_count->align(FL_ALIGN_RIGHT);
 	b_count->static_value("100");
+  
   y += b_count->h();
+  b_tbe = new Fl_Check_Button(b_yes->x(), y, 265, 20, "Transfer Bootstrap Expectation method");
+  b_tbe->deactivate();
+  y += b_tbe->h();
   b_keep_trees = new Fl_Check_Button(b_yes->x(), y, 180, 20, "Show bootstrap trees");
   b_keep_trees->deactivate();
+
   b_alrt->callback(support_cb, b_yes);
   b_no->callback(support_cb, b_yes);
   b_yes->callback(support_cb, b_yes);
@@ -1219,7 +1247,7 @@ void run_phyml_callback(Fl_Widget *ob, void *data)
         fclose(in);
       }
     }
-    if (replicates > 0 && b_keep_trees->value()) {
+    if (replicates > 0 && (b_keep_trees->value() || b_tbe->value())) {
       sprintf(input, "%s.phy_phyml_boot_trees.txt", base_fname);
       in = fopen(input, "r");
       if (!in) {
@@ -1271,18 +1299,47 @@ void run_phyml_callback(Fl_Widget *ob, void *data)
       }
     }
 	  //clean tree internal labels
-	  if(replicates > 0) display_tree = bootstrap_reformat(display_tree, replicates);
+    char *display_tree_tbe = NULL;
+    if(replicates > 0) {
+      if (b_tbe->value()){
+        char *tmp = strdup(display_tree);
+        display_tree_tbe = TBE_Compute(tmp, bootstrap_trees);
+      }
+      display_tree = bootstrap_reformat(display_tree, replicates);
+      if(bootstrap_trees && !b_keep_trees->value()) {
+        free(bootstrap_trees);
+        bootstrap_trees = NULL;
+      }
+    }
 	  else if(replicates < 0) display_tree = alrt_reformat(display_tree);
+	  
 	  //assemble tree description
 	  char tree_label[100];
 	  sprintf(tree_label, "PhyML ln(L)=%.1f %d sites %s", logL, seqlen, modelname);
-	  if(replicates > 0) sprintf(tree_label + strlen(tree_label), " %d replic.", replicates);
+	  if(replicates > 0) {
+      sprintf(tree_label + strlen(tree_label), " %d replic.", replicates);
+	  }
 	  if(cats > 1) sprintf(tree_label + strlen(tree_label), " %d rate classes", cats);
 	  if(user_tree) strcat(tree_label, " User-tree");
 	  //add label to tree
 	  char *tree = (char *)malloc(strlen(tree_label) + strlen(display_tree) + 4);
 	  sprintf(tree, "[%s] %s", tree_label, display_tree);
 	  free(display_tree);
+    
+    if (replicates > 0 && b_tbe->value()) {
+      //assemble TBE tree description
+      char tree_label_tbe[100];
+      sprintf(tree_label_tbe, "PhyML ln(L)=%.1f %d sites %s", logL, seqlen, modelname);
+      sprintf(tree_label_tbe + strlen(tree_label_tbe), " %d replic. (TBE)", replicates);
+      if(cats > 1) sprintf(tree_label_tbe + strlen(tree_label_tbe), " %d rate classes", cats);
+      if(user_tree) strcat(tree_label_tbe, " User-tree");
+      //add label to tree
+      char *tree = (char *)malloc(strlen(tree_label_tbe) + strlen(display_tree_tbe) + 4);
+      sprintf(tree, "[%s] %s", tree_label_tbe, display_tree_tbe);
+      free(display_tree_tbe);
+      display_tree_tbe = tree;
+    }
+    
 	  if (is_view_valid(view)) {
 	    const char *n = extract_filename(view->masename);
 	    char *title = NULL;
@@ -1293,13 +1350,19 @@ void run_phyml_callback(Fl_Widget *ob, void *data)
 	      sprintf(title, "%s-%s", n, "PhyML_tree");
 	      if (q) *q = '.';
 	    }
-      Fl_Window *btreew, *treew;
-      if (bootstrap_trees) {
+      Fl_Window *btreew=0, *treew=0;
+      if (replicates > 0 && b_keep_trees->value()) {
         btreew = treedraw(bootstrap_trees, view, "bootstrap trees", FALSE);
+      }
+      if (replicates > 0 && b_tbe->value()) {
+        const char *p = n ? title:"PhyML_tree";
+        char *q = (char*)malloc(strlen(p) + 5);
+        sprintf(q, "%s_TBE", p);
+        treedraw(display_tree_tbe, view, q, FALSE);
       }
 	    treew = treedraw(tree, view, n ? title:"PhyML_tree", FALSE);
 #if !defined(__APPLE__)
-      if (bootstrap_trees) treew->position(btreew->x() + 30, btreew->y() + 40);
+      if (treew && btreew) treew->position(btreew->x() + 30, btreew->y() + 40);
 #endif
 	    delete[] title;
 	  }
@@ -2133,7 +2196,8 @@ void parsimony_dialog(SEA_VIEW *view)
 	w_btrepl->align(FL_ALIGN_RIGHT);
   Fl_Check_Button *keep_b_trees = new Fl_Check_Button(2, w_dobootstrap->y() + w_dobootstrap->h() + 5, 180, 20, "Show bootstrap trees");
   keep_b_trees->deactivate();
-  w_dobootstrap->callback(dist_b_cb, keep_b_trees);
+  static Fl_Check_Button *buttons[2] = {keep_b_trees, NULL};
+  w_dobootstrap->callback(dist_b_cb, buttons);
   w_dobootstrap->when(FL_WHEN_CHANGED);
 
 	Fl_Box *w_count = new Fl_Box(0, keep_b_trees->y() + keep_b_trees->h() + 5, w->w(), 20, NULL);
@@ -2744,7 +2808,7 @@ void command_line_phylogeny(int argc, char **argv)
     
     view->tot_trees = 0;
     tree = run_distance_method(view, distkind, isarg(argc, argv, "-nogaps"), !isarg(argc, argv, "-NJ"), replicates > 1, replicates, NULL,
-			       distname, distkind == Ka || distkind == Ks, argname(argc, argv, "-distance_matrix"), utree, true, Fl::fatal, false);
+			       distname, distkind == Ka || distkind == Ks, argname(argc, argv, "-distance_matrix"), utree, true, Fl::fatal, false, false);
     if (isarg(argc, argv, "-distance_matrix")) exit(0);
   }
   
