@@ -392,6 +392,12 @@ void draw_tree(FD_unrooted *ob)
 	int char_width, char_height, num, dernier, encore = FALSE, ascend, descend;
 	hit result;
 	hits_field hits;
+#if !(defined(__APPLE__) || defined(WIN32))
+        if (fl_display && Fl::event() != FL_DRAG) {
+          ob->unrooted_plot->window()->cursor(FL_CURSOR_WAIT);
+          Fl::flush();
+        }
+#endif
 	char_width = myrint(fl_width("M"));
 	char_height = my_get_char_height(&ascend, &descend);
 	dernier = 2*ob->notu-3;
@@ -424,6 +430,9 @@ void draw_tree(FD_unrooted *ob)
 				  descend, ascend);
 	}
 	if(ob->has_br_length) draw_scale(ob, char_height, ascend);
+#if !(defined(__APPLE__) || defined(WIN32))
+        if (fl_display && Fl::event() != FL_DRAG) ob->unrooted_plot->window()->cursor(FL_CURSOR_DEFAULT);
+#endif
 }
 
 
@@ -613,17 +622,6 @@ hit try_angle_draw(float w, int *px, int *py, float *pdegrees /* clockwise degre
 	if(radians < 0) radians += 2 * M_PI;
 	float cosa = cos(radians);
 	float sina = sin(radians);
-	if(radians > M_PI/2 && radians < 3*M_PI/2 ) {//draw string so its end is at position x,y
-		x1 += (int)( w * cosa );
-		y1 -= (int)( w * sina );
-		radians += M_PI;
-		degrees -= 180;/* clockwise degrees */
-		if(radians >= 2*M_PI) radians -= 2 * M_PI;
-		if(degrees <= 0) degrees += 360;
-		cosa = - cosa;  sina = - sina;
-	}
-	x1 += (int)( (fl_size()/3) * sina );//position string midheight at position x,y
-	y1 += (int)( (fl_size()/3) * cosa );
 	
 	const float ch = 0.7;
 	x2 = x1 - (int)(ch * fl_size() * sina);
@@ -652,15 +650,32 @@ hit draw_name_angle(cp_point *phys_pos, const char *nom, FD_unrooted *fd_unroote
 					int ascent)
 {	
 	int x, y;
-	float degrees = - radians * 180/M_PI;
+	float degrees = - int(radians * 180/M_PI);
 	x = (int)phys_pos->x;
 	y = (int)phys_pos->y;
-	float w = fl_width(nom);
+        int dx, dy;
+        fl_measure(nom, dx, dy);
+        float w = dx, l = dy/4., fdx;
 	hit result = try_angle_draw(w, &x, &y, &degrees, fd_unrooted->x,  fd_unrooted->y,  fd_unrooted->w,  fd_unrooted->h);
 	if( (!doit) && result != NO_HIT) return result;
 	if( ! doit) return NO_HIT;
 	if(color) fl_color(FL_RED);
-	fl_draw(-(int)degrees, nom, x, y);
+        int rt_angle = -(int)degrees;
+        if (rt_angle <= 90 || rt_angle >= 270) {
+          if (rt_angle >= 270) rt_angle -= 360;
+          fdx =  l * sin(M_PI*rt_angle/180.);
+          dx = (int)fdx;
+          dy =  sqrt(l*l - fdx*fdx);
+          fl_draw(rt_angle, nom, x + dx, y + dy);
+        } else {
+          rt_angle -= 180;
+          float ddy = -sin(M_PI*(rt_angle+180)/180.) * dx;
+          float ddx = cos(M_PI*(rt_angle+180)/180.) * dx;
+          fdx = l * sin(M_PI*rt_angle/180.);
+          dx = fdx + ddx;
+          dy = sqrt(l*l - fdx*fdx) + ddy;
+          fl_draw(rt_angle, nom, x + dx, y + dy);
+        }
 	if(color) fl_color(FL_BLACK);
 	return NO_HIT;
 }
