@@ -1,12 +1,13 @@
 #ifdef WIN32
+
+#include <FL/Enumerations.H>
 #include "pdf_or_ps.h"
-#include <Windows.h>
 #include <FL/fl_utf8.h>
 #include <stdlib.h>
 #include <stdio.h>
-#ifndef NO_PDF
-#include <pdflib.h>
-#endif
+#  ifndef NO_PDF
+#    include <pdflib.h>
+#  endif
 
 /* included functions */
 void my_bell(void);
@@ -14,6 +15,7 @@ char *get_prog_dir(void);
 char **getutf8args(int *pargc);
 int mysystem(const char *cmd);
 
+#if 100*FL_MAJOR_VERSION + 10*FL_MINOR_VERSION + FL_PATCH_VERSION < 134 // FLTK before 1.3.4
 
 Copy_Surface::Copy_Surface(int w, int h) : Fl_Surface_Device(new Fl_GDI_Graphics_Driver())
 {
@@ -57,6 +59,8 @@ Copy_Surface::~Copy_Surface()
   delete driver();
 }
 
+#endif // FLTK before 1.3.4
+
 
 char *get_prog_dir(void)
 /* returns path of dir where calling program was launched */
